@@ -1,5 +1,5 @@
 from uvicorn.config import Config
-from uvicorn.main import main, run, Server
+from uvicorn.main import Server, main, run
 
-__version__ = "0.4.4"
+__version__ = "0.5.2"
 __all__ = ["main", "run", "Config", "Server"]
