@@ -2,12 +2,17 @@ import asyncio
 from urllib.parse import unquote
 
 import h11
-import wsproto.connection
-import wsproto.events
-import wsproto.extensions
+import wsproto
+from wsproto import ConnectionType, events
+from wsproto.connection import ConnectionState
+from wsproto.extensions import PerMessageDeflate
+from wsproto.utilities import RemoteProtocolError
 
 from uvicorn.protocols.utils import get_local_addr, get_remote_addr, is_ssl
 
+# Check wsproto version. We've build against 0.13. We don't know about 0.14 yet.
+assert wsproto.__version__ > "0.13", "Need wsproto version 0.13"
+
 
 class WSProtocol(asyncio.Protocol):
     def __init__(self, config, server_state, _loop=None):
@@ -36,10 +41,7 @@ class WSProtocol(asyncio.Protocol):
         self.handshake_complete = False
         self.close_sent = False
 
-        self.conn = wsproto.connection.WSConnection(
-            conn_type=wsproto.connection.SERVER,
-            extensions=[wsproto.extensions.PerMessageDeflate()],
-        )
+        self.conn = wsproto.WSConnection(connection_type=ConnectionType.SERVER)
 
         self.read_paused = False
         self.writable = asyncio.Event()
@@ -59,25 +61,40 @@ class WSProtocol(asyncio.Protocol):
         self.scheme = "wss" if is_ssl(transport) else "ws"
 
     def connection_lost(self, exc):
+        if exc is not None:
+            self.queue.put_nowait({"type": "websocket.disconnect"})
         self.connections.remove(self)
 
     def eof_received(self):
         pass
 
     def data_received(self, data):
-        self.conn.receive_bytes(data)
+        try:
+            self.conn.receive_data(data)
+        except RemoteProtocolError as err:
+            if err.event_hint is not None:
+                self.transport.write(self.conn.send(err.event_hint))
+                self.transport.close()
+            else:
+                self.handle_no_connect(events.CloseConnection())
+        else:
+            self.handle_events()
+
+    def handle_events(self):
         for event in self.conn.events():
-            if isinstance(event, wsproto.events.ConnectionRequested):
+            if isinstance(event, events.Request):
                 self.handle_connect(event)
-            elif isinstance(event, wsproto.events.TextReceived):
+            elif isinstance(event, events.TextMessage):
                 self.handle_text(event)
-            elif isinstance(event, wsproto.events.BytesReceived):
+            elif isinstance(event, events.BytesMessage):
                 self.handle_bytes(event)
-            elif isinstance(event, wsproto.events.ConnectionFailed):
+            elif isinstance(event, events.RejectConnection):
                 self.handle_no_connect(event)
-            elif isinstance(event, wsproto.events.ConnectionClosed):
+            elif isinstance(event, events.RejectData):
+                self.handle_no_connect(event)
+            elif isinstance(event, events.CloseConnection):
                 self.handle_close(event)
-            elif isinstance(event, wsproto.events.PingReceived):
+            elif isinstance(event, events.Ping):
                 self.handle_ping(event)
 
     def pause_writing(self):
@@ -94,8 +111,7 @@ class WSProtocol(asyncio.Protocol):
 
     def shutdown(self):
         self.queue.put_nowait({"type": "websocket.disconnect", "code": 1012})
-        self.conn.close(1012)
-        output = self.conn.bytes_to_send()
+        output = self.conn.send(wsproto.events.CloseConnection(code=code))
         self.transport.write(output)
         self.transport.close()
 
@@ -106,19 +122,20 @@ class WSProtocol(asyncio.Protocol):
 
     def handle_connect(self, event):
         self.connect_event = event
-        request = event.h11request
-        headers = [(key.lower(), value) for key, value in request.headers]
-        path, _, query_string = request.target.partition(b"?")
+        headers = [(b"host", event.host.encode())]
+        headers += [(key.lower(), value) for key, value in event.extra_headers]
+        path, _, query_string = event.target.partition("?")
         self.scope = {
             "type": "websocket",
+            "http_version": "1.1",
             "scheme": self.scheme,
             "server": self.server,
             "client": self.client,
             "root_path": self.root_path,
-            "path": unquote(path.decode("ascii")),
-            "query_string": query_string,
+            "path": unquote(path),
+            "query_string": query_string.encode("ascii"),
             "headers": headers,
-            "subprotocols": [],
+            "subprotocols": event.subprotocols,
         }
         self.queue.put_nowait({"type": "websocket.connect"})
         task = self.loop.create_task(self.run_asgi())
@@ -131,11 +148,11 @@ class WSProtocol(asyncio.Protocol):
             (b"connection", b"close"),
         ]
         msg = h11.Response(status_code=400, headers=headers, reason="Bad Request")
-        output = self.conn._upgrade_connection.send(msg)
+        output = self.conn.send(msg)
         msg = h11.Data(data=event.reason.encode("utf-8"))
-        output += self.conn._upgrade_connection.send(msg)
+        output += self.conn.send(msg)
         msg = h11.EndOfMessage()
-        output += self.conn._upgrade_connection.send(msg)
+        output += self.conn.send(msg)
         self.transport.write(output)
         self.transport.close()
 
@@ -150,6 +167,7 @@ class WSProtocol(asyncio.Protocol):
 
     def handle_bytes(self, event):
         self.bytes += event.data
+        # todo: we may want to guard the size of self.bytes and self.text
         if event.message_finished:
             self.queue.put_nowait({"type": "websocket.receive", "bytes": self.bytes})
             self.bytes = b""
@@ -158,26 +176,30 @@ class WSProtocol(asyncio.Protocol):
                 self.transport.pause_reading()
 
     def handle_close(self, event):
+        if self.conn.state == ConnectionState.REMOTE_CLOSING:
+            self.transport.write(self.conn.send(event.response()))
         self.queue.put_nowait({"type": "websocket.disconnect", "code": event.code})
         self.transport.close()
 
     def handle_ping(self, event):
-        output = self.conn.bytes_to_send()
-        self.transport.write(output)
+        self.transport.write(self.conn.send(event.response()))
 
     def send_500_response(self):
         headers = [
             (b"content-type", b"text/plain; charset=utf-8"),
             (b"connection", b"close"),
         ]
-        msg = h11.Response(
-            status_code=500, headers=headers, reason="Internal Server Error"
-        )
-        output = self.conn._upgrade_connection.send(msg)
-        msg = h11.Data(data=b"Internal Server Error")
-        output += self.conn._upgrade_connection.send(msg)
-        msg = h11.EndOfMessage()
-        output += self.conn._upgrade_connection.send(msg)
+        if self.conn.connection is None:
+            output = self.conn.send(wsproto.events.RejectConnection(status_code=500))
+        else:
+            msg = h11.Response(
+                status_code=500, headers=headers, reason="Internal Server Error"
+            )
+            output = self.conn.send(msg)
+            msg = h11.Data(data=b"Internal Server Error")
+            output += self.conn.send(msg)
+            msg = h11.EndOfMessage()
+            output += self.conn.send(msg)
         self.transport.write(output)
 
     async def run_asgi(self):
@@ -215,8 +237,11 @@ class WSProtocol(asyncio.Protocol):
                 )
                 self.handshake_complete = True
                 subprotocol = message.get("subprotocol")
-                self.conn.accept(self.connect_event, subprotocol)
-                output = self.conn.bytes_to_send()
+                output = self.conn.send(
+                    wsproto.events.AcceptConnection(
+                        subprotocol=subprotocol, extensions=[PerMessageDeflate()]
+                    )
+                )
                 self.transport.write(output)
 
             elif message_type == "websocket.close":
@@ -229,9 +254,9 @@ class WSProtocol(asyncio.Protocol):
                 self.handshake_complete = True
                 self.close_sent = True
                 msg = h11.Response(status_code=403, headers=[])
-                output = self.conn._upgrade_connection.send(msg)
+                output = self.conn.send(msg)
                 msg = h11.EndOfMessage()
-                output += self.conn._upgrade_connection.send(msg)
+                output += self.conn.send(msg)
                 self.transport.write(output)
                 self.transport.close()
 
@@ -244,8 +269,7 @@ class WSProtocol(asyncio.Protocol):
                 bytes_data = message.get("bytes")
                 text_data = message.get("text")
                 data = text_data if bytes_data is None else bytes_data
-                self.conn.send_data(data)
-                output = self.conn.bytes_to_send()
+                output = self.conn.send(wsproto.events.Message(data=data))
                 if not self.transport.is_closing():
                     self.transport.write(output)
 
@@ -253,8 +277,7 @@ class WSProtocol(asyncio.Protocol):
                 self.close_sent = True
                 code = message.get("code", 1000)
                 self.queue.put_nowait({"type": "websocket.disconnect", "code": code})
-                self.conn.close(code)
-                output = self.conn.bytes_to_send()
+                output = self.conn.send(wsproto.events.CloseConnection(code=code))
                 if not self.transport.is_closing():
                     self.transport.write(output)
                     self.transport.close()
